from __future__ import print_function
import wx
import sys

if sys.version_info.major > 2:
    basestring = str
    map = lambda a, b: [a(_b) for _b in b]

LABEL_ARGS = dict()  # Arguments used for wx.StaticText
CTRL_ARGS = dict()  # Arguments used for wx.<Control>

try:
    import poser
except ImportError:
    import POSER_FAKE as poser


aui = poser.WxAuiManager()
root = aui.GetManagedWindow()
SYS_FONT = root.GetFont()
CURRENT_GRB_MGR = None
GLOBAL_PARENTWINDOW = None
CONFIG = dict(BackgroundColor=wx.SYS_COLOUR_BACKGROUND,
              ForegroundColor= 0x40 * 3)


class GridBagManager(object):
    __slots__ = "parent", "widgets", "sizer", "flag", "border", "current_row", "font"

    def __init__(self, *args, **kwargs):
        global CURRENT_GRB_MGR, GLOBAL_PARENTWINDOW

        for varname in self.__slots__:
            setattr(self, varname, kwargs.get(varname, None))

        for idx, entry in enumerate(args):
            if self.__slots__[idx] not in kwargs:
                setattr(self, self.__slots__[idx], entry)

        self.current_row = 0
        self.widgets = []
        CURRENT_GRB_MGR = self
        GLOBAL_PARENTWINDOW = self.parent

    def addrow(self, *args, **kwargs):
        row = int(kwargs.get("row", self.current_row))
        col = int(kwargs.get("startcol", 0))
        flag = kwargs.get("flag", self.flag)
        border = kwargs.get("border", self.border)
        font = kwargs.pop("font", SYS_FONT)
        sz = self.sizer  # type: wx.GridBagSizer
        for idx, widget in enumerate(args):
            if widget is None:
                sz.AddGrowableRow(row)
            else:
                if font:
                    widget.SetFont(font)

                self.sizer.Add(widget, pos=(row, col + idx),
                               flag=flag,
                               border=
                               border)
                if widget.__class__ != wx.StaticText:
                    self.widgets.append(widget)

        self.current_row += 1

    def get_items_by_name(self, name, filter=None):
        if not filter:
            return [w for w in self.widgets if w.GetName().startswith(name)]
        return [w for w in self.widgets if w.GetName().startswith(name)
                and w.__class__.__name__ in filter]

    def get_value_by_name(self, name, filter=None):
        obj = self.get_items_by_name(name, filter)
        if not obj:
            return None
        if len(obj) == 1:
            return obj[0].Value()
        return [o.Value() for o in obj]

    def items_as_dict(self, filter=None):
        d = dict()
        for widget in [w for w in self.widgets
                       if filter is None or w.__class__.__name__ in filter]:
            if hasattr(widget, "GetCheckedStrings"):
                d[widget.GetName()] = widget.GetCheckedStrings()
            elif hasattr(widget, "GetValue"):
                d[widget.GetName()] = widget.GetValue()
            elif hasattr(widget, "GetSelection"):
                v = widget.GetSelection()
                if hasattr(widget, "GetString"):
                    v = widget.GetString(v)
                d[widget.GetName()] = v

        return d

    def addrows(self, *rows2add):
        for row in rows2add:
            if row is None:
                self.addrow(None)
            else:
                self.addrow(*row)

    def setStyles(self, ctrltype, **kwargs):
        for c in self.widgets:
            if c.__class__ is ctrltype:
                c.SetWindowStyle(**kwargs)

    def bindCtrls(self, ctrltype, ev, target):
        for c in self.widgets:
            if c.__class__ is ctrltype:
                c.Bind(ev, target)


def setTooltip(ctrl, text):
    if text and ctrl:
        t = wx.ToolTip(text)
        t.SetAutoPop(5000)
        ctrl.SetToolTip(t)
    return ctrl


def ColoredCtrl(ctrl, **kwargs):
    ctrl.SetBackgroundColour(kwargs.pop("bgcolor", CONFIG.get("BackgroundColor")))
    ctrl.SetForegroundColour(kwargs.pop("fgcolor", CONFIG.get("ForegroundColor")))
    return setTooltip(ctrl, kwargs.pop("tooltip", None))


def LabelCtrl(**kwargs):
    parent = GLOBAL_PARENTWINDOW
    bg_color = kwargs.pop("bgcolor", CONFIG.get("BackgroundColor"))
    fg_color = kwargs.pop("fgcolor", CONFIG.get("ForegroundColor"))
    font = kwargs.pop("font", None)
    tooltip = kwargs.pop("tooltip", None)
    ctrl = wx.StaticText(parent, **kwargs)
    if font:
        ctrl.SetFont(font)
    ctrl.SetBackgroundColour(bg_color)
    ctrl.SetForegroundColour(fg_color)
    return setTooltip(ctrl, tooltip)


def PrepCtrl(ctrl, **kwargs):
    parent = GLOBAL_PARENTWINDOW
    bind = kwargs.pop("bind", [])
    fgcolor = kwargs.pop("fgcolor", CONFIG.get("ForegroundColor"))
    bgcolor = kwargs.pop("bgcolor", CONFIG.get("BackgroundColor"))
    font = kwargs.pop("font", None)
    value = kwargs.pop("value", None)
    index = kwargs.pop("index", None)
    tooltip = kwargs.pop("tooltip", None)
    if "key" in kwargs:
        kwargs["name"] = kwargs["key"]
        kwargs.pop("key")
    ctrl = ctrl(parent, **kwargs)

    if bind:
        if isinstance(bind[0], (tuple, list)):
            for bind_entry in bind:
                assert len(bind_entry) == 2 and callable(bind_entry[1]), \
                    "Bind entries must be: int(EVT_ID), func<callable>"
                ctrl.Bind(bind_entry[0], bind_entry[1])
        else:
            assert len(bind) == 2 and callable(bind[1]), \
                "Bind entries must be: int(EVT_ID), func<callable>"
            ctrl.Bind(bind[0], bind[1])

    ctrl.SetForegroundColour(fgcolor)
    ctrl.SetBackgroundColour(bgcolor)
    if font:
        ctrl.SetFont(font)
    if value:
        ctrl.SetValue(value)
    if index:
        ctrl.SetSelection(index)
    return setTooltip(ctrl, tooltip)


def Labeled(ctrl, tooltip=None, **kwargs):
    label_args = LABEL_ARGS.copy()
    tx = kwargs.pop("text", kwargs.pop("label", ""))
    if tx:
        label_args["label"] = tx
    ctrl_args = CTRL_ARGS.copy()
    for k in kwargs.keys():
        if k.startswith("lbl_"):
            label_args[k[4:]] = kwargs.pop(k)
        elif k.startswith("ctl_"):
            ctrl_args[k[4:]] = kwargs.pop(k)
    ctrl_args.update(kwargs)

    return (setTooltip(LabelCtrl(**label_args), tooltip),
            setTooltip(PrepCtrl(ctrl=ctrl, **ctrl_args), tooltip))


def ButtonRow(*buttons, **sizerflags):
    parent = GLOBAL_PARENTWINDOW
    sz = wx.BoxSizer(wx.LI_HORIZONTAL)
    for btn in buttons:
        if isinstance(btn, basestring):
            btn = wx.Button(parent, -1, btn)
        sz.Add(btn, **sizerflags)

    sz.Layout()
    return sz


def PopUpMessage(text, bg=(0xf0, 0xf0, 0xf0), fg=(0x10, 0x10, 0x10), closeafter=3):
    root = wx.GetTopLevelWindows()[0]
    dialog = wx.Frame(root, id=wx.ID_ANY, title="IM-Export Notification",
                      style=wx.CAPTION | wx.STAY_ON_TOP)
    dialog.SetForegroundColour(fg)
    dialog.SetBackgroundColour(bg)
    sz = wx.BoxSizer(wx.HORIZONTAL)
    txt = wx.StaticText(dialog, label=text)
    sz.Add(txt, 1, wx.ALL | wx.EXPAND, border=15)
    sz.Layout()
    dialog.SetSizer(sz)

    w, h = sz.ComputeFittingWindowSize(dialog)
    dialog.SetSize((w, h))

    pos_x, pos_y = root.GetPosition()
    size = root.GetSize()
    center_x = size[0] / 2.0
    center_y = size[1] / 2.0
    dialog.SetPosition((pos_x + center_x - w / 2, pos_y + center_y - h / 2))
    dialog.Show()

    if closeafter:
        wx.CallLater(closeafter * 1000, dialog.Destroy)

    return dialog
